/*
 * Copyright 2009, Josh Devins, Jitr.org.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.jitr.container;

import java.lang.annotation.Annotation;

import javax.servlet.ServletContext;

import org.apache.catalina.Engine;
import org.apache.catalina.Host;
import org.apache.catalina.LifecycleException;
import org.apache.catalina.Server;
import org.apache.catalina.connector.Connector;
import org.apache.catalina.core.StandardContext;
import org.apache.catalina.startup.Embedded;
import org.jitr.core.Container;
import org.jitr.core.ContainerOperationException;
import org.jitr.core.ConfigurationModel;

/**
 * Tomcat specific container wrapper.
 * 
 * @author Josh Devins (joshdevins@jitr.org)
 */
public class TomcatContainer implements Container<Server> {

    private static final String NAME = "Tomcat";

    private static final String LOCALHOST = "localhost";

    private final Embedded embedded;

    private final Host host;

    public TomcatContainer() {
        embedded = new Embedded();
        embedded.setName(NAME);
        embedded.setCatalinaBase("");

        // create engine
        Engine engine = embedded.createEngine();
        engine.setDefaultHost(LOCALHOST);
        embedded.addEngine(engine);

        // add host with default webapps path
        host = embedded.createHost(LOCALHOST, "");
        engine.addChild(host);
    }

    public String getName() {
        return NAME;
    }

    public Server getWrappedContainer() {
        return embedded.getServer();
    }

    public ServletContext initialize(final ConfigurationModel configuration,
            final Annotation[] testClassAnnotations) {

        // add connector
        Connector connector = embedded.createConnector(LOCALHOST, configuration.getPort(), false);
        connector.setUseBodyEncodingForURI(true);
        connector.setURIEncoding("UTF-8");
        embedded.addConnector(connector);

        // add specific war context
        StandardContext context =
                (StandardContext) embedded.createContext(configuration.getContextPath(),
                        configuration.getWarPath());
        context.setReloadable(false);
        host.addChild(context);

        context.setWorkDir(configuration.getContainerWorkPath());

        return context.getServletContext();
    }

    public void start() throws ContainerOperationException {
        try {
            embedded.start();
        } catch (final LifecycleException le) {
            throw new ContainerOperationException("Error starting Tomcat.", le, this);
        }
    }

    public void stop() throws ContainerOperationException {
        try {
            embedded.stop();
        } catch (final LifecycleException le) {
            throw new ContainerOperationException("Error stopping Tomcat.", le, this);
        }
    }
}
