/*
 * Copyright 2009, Josh Devins, Jitr.org.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.jitr.core;

/**
 * Exception specific to container operations. This is a general wrapper for all container specific
 * exceptions.
 * 
 * @author Josh Devins (joshdevins@jitr.org)
 */
public class ContainerOperationException extends JitrException {

    private static final long serialVersionUID = -1087569754991543424L;

    private final String containerName;

    public ContainerOperationException(final String message, final Container<?> container) {
        super(message);

        containerName = container.getName();
    }

    public ContainerOperationException(final String message, final Throwable cause,
            final Container<?> container) {
        super(message, cause);

        containerName = container.getName();
    }

    public String getContainerName() {
        return containerName;
    }
}
